#include <owl\applicat.h>
#include <owl\framewin.h>
#include <owl\gdiobjec.h>
#include <owl\dc.h>
#include <owl\color.h>
#include <stdlib.h>
#pragma hdrstop
#include "transbmp.rh"

const int ballColor[5][3] = {		//colors used for the red ball
	{181,33,33},
	{255,99,66},
	{255,165,0},
	{255,214,0},
	{255,255,206}
};

const int COLORSIZE = 236;	//number of colors used (20 are reserved)
const int BALLCOLORS = 5;			//number of colors for the red ball
const int RNDTOP = 30;
const int RNDBOT = 10;
const int BITMAPW = 20;
const int BITMAPH = 20;


class TWndw : public TFrameWindow
{
public:
	TWndw(TWindow *parent, const char far *title);
	~TWndw();
protected:
	void SetupWindow();
	void CleanupWindow();
	BOOL EvEraseBkgnd(HDC hDC);
	void EvTimer(UINT);
	void Paint(TDC& dc, BOOL erase, TRect& rect);
	void EvSize(UINT sizeType, TSize& size);
private:
	void InitCoords();
	void GetNewPosition();
	void MoveBall();

	TBitmap *foreBitmap, *backBitmap;
	TMemoryDC *screenBackup;
	TPalette *palette;
	TRect clientRect;
	TRect ballPos, oldPos;
	TSize delta;
DECLARE_RESPONSE_TABLE(TWndw);
};

DEFINE_RESPONSE_TABLE1(TWndw, TFrameWindow)
	EV_WM_ERASEBKGND,
	EV_WM_SIZE,
	EV_WM_TIMER,
END_RESPONSE_TABLE;


TWndw::TWndw(TWindow *parent, const char far *title)
	:	TFrameWindow(parent, title),
		screenBackup(0)
{
	AssignMenu(MENU_1);

	TPaletteEntry* palEntries;
	palEntries = (TPaletteEntry*)new char[sizeof(TPaletteEntry) * COLORSIZE];

	for (int i = 0; i < COLORSIZE - BALLCOLORS; i++) {
		palEntries[i].peRed = 0;
		palEntries[i].peGreen = 0;
		palEntries[i].peBlue = COLORSIZE - i - 1;
		palEntries[i].peFlags = PC_RESERVED;
	}

	for (i = COLORSIZE - BALLCOLORS; i < COLORSIZE; i++) {
		palEntries[i].peRed = ballColor[i - COLORSIZE + BALLCOLORS][0];
		palEntries[i].peGreen = ballColor[i - COLORSIZE + BALLCOLORS][1];
		palEntries[i].peBlue = ballColor[i - COLORSIZE + BALLCOLORS][2];
		palEntries[i].peFlags = PC_RESERVED;
	}

	palette = new TPalette(palEntries, COLORSIZE);
	delete[] palEntries;

	foreBitmap = new TBitmap(*GetModule(), BITMAP_REDBALL);
	backBitmap = new TBitmap(*GetModule(), BITMAP_BACK);
	randomize();		//life without it is boring
}


TWndw::~TWndw()
{
	delete foreBitmap;
	delete backBitmap;
	delete screenBackup;
	delete palette;
}


void TWndw::SetupWindow()
{
	TFrameWindow::SetupWindow();
	SetTimer(1, 100);
}


void TWndw::InitCoords()
{
	int startX = random(clientRect.right - BITMAPW);
	int startY = random(clientRect.bottom - BITMAPH);
	ballPos = TRect(startX, startY, startX + BITMAPW, startY + BITMAPH);
	oldPos = ballPos;		//initially old position is same as ball position
	delta = TSize(random(RNDTOP) + RNDBOT, random(RNDTOP) + RNDBOT);

	TClientDC clientDC(*this);
	clientDC.SelectObject(*palette);
	clientDC.RealizePalette();

	delete screenBackup;
	screenBackup = new TMemoryDC(clientDC);
	screenBackup->SelectObject(*palette);

	TBitmap screenBitmap(clientDC, BITMAPW, BITMAPH);
	screenBackup->SelectObject(screenBitmap);
	screenBackup->BitBlt(0, 0, BITMAPW, BITMAPH, clientDC,
		ballPos.left, ballPos.top, SRCCOPY);

	clientDC.RestorePalette();
	screenBackup->RestorePalette();
}


void TWndw::CleanupWindow()
{
	KillTimer(1);
}


void TWndw::EvTimer(UINT)
{
	GetNewPosition();
	MoveBall();
}


void TWndw::GetNewPosition()
{
	oldPos = ballPos;
	ballPos += delta;

	if (ballPos.left < 0) {
		ballPos.Offset(-ballPos.left, 0);
		delta.cx = random(RNDTOP) + RNDBOT;
	}

	if (ballPos.top < 0) {
		ballPos.Offset(0, -ballPos.top);
		delta.cy = random(RNDTOP) + RNDBOT;
	}

	if (ballPos.right > clientRect.right) {
		ballPos.Offset(-(ballPos.right - clientRect.right), 0);
		delta.cx = -(random(RNDTOP) + RNDBOT);
	}

	if (ballPos.bottom > clientRect.bottom) {
		ballPos.Offset(0, -(ballPos.bottom - clientRect.bottom));
		delta.cy = -(random(RNDTOP) + RNDBOT);
	}
}


void TWndw::MoveBall()
{
	TClientDC clientDC(*this);
	clientDC.SelectObject(*palette);
	clientDC.RealizePalette();

	//restore old background
	clientDC.BitBlt(oldPos, *screenBackup, TPoint(0, 0), SRCCOPY);
	screenBackup->SelectObject(*palette);

	//save new position's background
	TBitmap sb(clientDC, BITMAPW, BITMAPH);
	screenBackup->SelectObject(sb);
	screenBackup->BitBlt(0, 0, BITMAPW, BITMAPH, clientDC,
		ballPos.left, ballPos.top, SRCCOPY);

	TMemoryDC tempMemDC(clientDC);		//temp mem for loading images
	tempMemDC.SelectObject(*palette);
	TMemoryDC memDC(clientDC);				//scratch pad for image mix
	memDC.SelectObject(*palette);

	TBitmap screenBitmap(clientDC, BITMAPW, BITMAPH);
	memDC.SelectObject(screenBitmap);
	memDC.BitBlt(0, 0, BITMAPW, BITMAPH, clientDC,		//copy screen to bitmap
		ballPos.left, ballPos.top, SRCCOPY);

	tempMemDC.SelectObject(*backBitmap);		//select ball's backgdound
	memDC.BitBlt(0, 0, BITMAPW, BITMAPH, tempMemDC, 0, 0, SRCAND);		//AND
	tempMemDC.RestoreBitmap();

	tempMemDC.SelectObject(*foreBitmap);		//select the ball itself
	memDC.BitBlt(0, 0, BITMAPW, BITMAPH, tempMemDC, 0, 0, SRCPAINT);	//OR

	clientDC.BitBlt(ballPos, memDC, TPoint(0, 0), SRCCOPY);		//to screen

	clientDC.RestorePalette();
	memDC.RestorePalette();
	tempMemDC.RestorePalette();
	screenBackup->RestorePalette();
}


void TWndw::Paint(TDC& dc, BOOL /*erase*/, TRect& /*rect*/)
{
	char buf[50];
	wsprintf(buf, "Client Area : width = %d  height = %d",
		clientRect.Width(), clientRect.Height());
	dc.SetBkMode(TRANSPARENT);
	dc.SetTextColor(TColor::White);
	dc.SetTextAlign(TA_CENTER);
	dc.SelectObject(TFont("Times New Roman", 20));
	dc.TextOut(clientRect.right / 2, clientRect.bottom / 2, buf);

	InitCoords();
}


void TWndw::EvSize(UINT sizeType, TSize& size)
{
	TFrameWindow::EvSize(sizeType, size);
	Invalidate();
	UpdateWindow();		//not strictly necessary
}


BOOL TWndw::EvEraseBkgnd(HDC hDC)
{
	TDC dc(hDC);
	dc.SelectObject(*palette);
	dc.RealizePalette();

	clientRect = GetClientRect();
	TRect rcFill(0, 0, clientRect.right,
		MulDiv(1, clientRect.bottom, COLORSIZE - BALLCOLORS - 1));

	for (int i = 0; i < COLORSIZE - BALLCOLORS - 1; i++) {
		dc.FillRect(rcFill, TBrush(TColor(i)));
		rcFill.top = rcFill.bottom;
		rcFill.bottom = MulDiv(i + 2, clientRect.bottom,
			COLORSIZE - BALLCOLORS - 1);
	}

	dc.RestorePalette();
	return TRUE;
}

///////////////////////////////////////////////////////////////////////////
class TApp : public TApplication
{
public:
	TApp(): TApplication() {}
	void InitMainWindow();
};


void TApp::InitMainWindow()
{
	TFrameWindow *wndw = new TWndw(0, "Transparent Background Bitmap");
	wndw->SetIcon(this, ICON_1);
	SetMainWindow(wndw);

	nCmdShow=SW_SHOWMAXIMIZED;
}


int OwlMain(int, char*[])
{
	return TApp().Run();
}

